/* Generated by Together */

#ifndef MEMMANAGER_H
#define MEMMANAGER_H

#include <vector>

#include "MemSpace.h"
#include "MemSorter.h"
#include "MemData.h"
#include "MemSpace.h"
#include "MemKey.h"

using namespace std;

//! Shared memory pool manager
/*!
 * This class manages a pool of MemSpace, it make the link between the MemKey and the MemSpace. It also creates the MemKey Only one instance of the manager can exist at one time
 */
class MemManager {
public:

	//! Destructor
	/*!
	 * Disposes cleanly of the manager
	 */
    ~MemManager();

    //! Method to get the instance of the Manager.
    /*!
     * If it does not exist, this method will create it
     * 
     * \return an instance of the manager
     */
    MemManager* getInstance();

    //! Initialise a shared memory space with the specified size
    /*!
     * This method creates a new shared memory space and a new key.
     * 
     * \return a MemKey that identifies the MemSpace and is necessary for future access
     */
    const MemKey* initSpace(int size);

    //! Initialise a shared memory space with the specified info
    /*!
     * This method creates a new shared memory space and create a new key or links to an existing one and returns the corresponding MemKey. To link 2 spaces, the size and the key muste be the same.
     * 
     * \param key the key identifiying the shared memory space
     * \param size the size of the shared memory space
     * \return a MemKey linking to a MemSpace dans that is needed for future access
     */
    const MemKey* initSpace(int key, int size);

    //! Initialise a shared memory space with the specified info
    /*!
     * This method creates a new shared memory space and create a new key or links to an existing one and returns the corresponding MemKey. To link 2 spaces, the size and the key muste be the same.
     * 
     * \param key the key identifiying the shared memory space
     * \param size the size of the shared memory space
     * \return a MemKey linking to a MemSpace dans that is needed for future access
     */
    const MemKey* initSpace(MemKey* key, int size);

    
    //! Sets the default MemSpace to work with
    /*!
     * \param key the key identifiying the MemSpace
     * \return a bool representing the succes of the operation
     */
    bool setDefaultSpace(MemKey* key);

    //! Sets the next MemSpace in the pool as the MemSpace to work with
    /*!
     * Works in a cirular maner, at the end of the pool the first one is referenced.
     */
    void nextSpace();

    //! Sets the previous MemSpace in the pool as the MemSpace to work with
    /*!
     * Works in a cirular maner, at the begining of the pool the last one is referenced.
     */
    void previousSpace();

    //! Returns the MemData from the current MemSpace
    /*!
     * \return the data from the MemSpace
     */
    MemData* fetchData( );

    //! Returns the MemData from the MemSpace with the specifed key
    /*!
     * \param key the key of the MemSpace
     * \return the data from the MemSpace
     */
    MemData* fetchData(int key);

    //! Returns the MemData from the MemSpace with the specifed key
    /*!
     * \param key the key of the MemSpace
     * \return the data from the MemSpace
     */
    MemData* fetchData(MemKey* key);

    //! Changes the data in the current MemSpace
    /*!
     * \param Data a pointer to the new data
     * \param size the size of Data
     * \return the success of the operation
     */
    bool putData(void * Data, int size);

    //! Changes the data in the MemSpace specified by the key
    /*!
     * \param key the key of the MemSpace
     * \param Data a pointer to the new data
     * \param size the size of Data
     * \return the success of the operation
     */
    bool putData(int key, void * Data, int size);

    //! Changes the data in the MemSpace specified by the key
    /*!
     * \param key the key of the MemSpace
     * \param Data a pointer to the new data
     * \param size the size of Data
     * \return the success of the operation
     */
    bool putData(MemKey* key, void * Data, int size);

    //! Gets a list of the available MemKeys
    /*!
     * \return a vector of available MemKey
     */
    vector<MemKey*> getAvailSpaces() const;


private:
	
	//! Generates a random key to access the MemSpace
	/*!
	 * \return the generated key
	 */
    int genKey();
    
    //! The MemSpaces
    vector<MemSpace*> spaces;
    
    //! The instance of the manager
    static MemManager* instance;
    
    // The current index of the pool of MemSpace
    int defaultIndex;
    
protected:
	
	//! Default consrtuctor
	/*!
	 * Part of a singleton patern
	 */
    MemManager();
    
};
#endif //MEMMANAGER_H
