/* Generated by Together */

#ifndef VIDEOINPUT_H
#define VIDEOINPUT_H

#include <semaphore.h>
#include "TimeInfo.h"

//!  Video input buffer
/*!
 * This class is the video input buffer of a mixer InputStreams
 */

class VideoInput {
public:

	//! Default Constructor
	/*!
	 * 	Initialise the buffer
	 */
    VideoInput();

    //! Destructor
    /*
     *	Cleanly disposes of the buffer
     */
    ~VideoInput();

    //! Access the data contained the the buffer
    /*!
     * This method returns the data. Buffer access is protected to prevent simultanious access.
     * 
     * \param data a pointer to where the data must be copied
     * \return the size of the fetched data 
     */
    virtual int fetchData(char* data) const;

    //! Access the timing information
    /*!
     * This method returns the timestamp for the data contained in the buffer. The data is used by the synchonization manager to mix the streams 
     */
    virtual TimeInfo fetchTimeInfo() const;

    //! Changes the data contained in the buffer
    /*!
     * This method changes the data contained in the buffer. Buffer access is protected to prevent simultanious access.
     * 
     * \param data a pointer to a data buffer
     * \param size the size of the buffer
     */
    virtual void putData(char * data, int size);

    void putTimeInfo(TimeInfo* infos);

private:
	
	//! Simultanious access protection semaphore 
    sem_t semaphore;
    
    //! Internal data buffer
    char * buffer;
    TimeInfo* infoTemps;    
};
#endif //VIDEOINPUT_H
